<?xml version="1.0" encoding="utf-8"?>
<!-- libfo -->
<!-- XSLT stylesheet to bring an XSL FO document into compatibility
     with the capabilities of the libfo library of the xmlroff XSL
     Formatter. -->
<!-- $Id: libfo-compat.xsl,v 1.2 2005/02/27 16:49:21 tonygraham Exp $ -->

<!-- Author: Tony Graham <tony.graham@sun.com> -->

<!--
     Copyright (c) 2003-2004 Sun Microsystems. All Rights Reserved.

     Redistribution and use in source and binary forms, with or without
     modification, are permitted provided that the following conditions are
     met:

      - Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.

      - Redistribution in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the
        distribution.

     Neither the name of Sun Microsystems or the names of contributors may
     be used to endorse or promote products derived from this software
     without specific prior written permission.

     This software is provided "AS IS," without a warranty of any kind. ALL
     EXPRESS OR IMPLIED CONDITIONS, REPRESENTATIONS AND WARRANTIES,
     INCLUDING ANY IMPLIED WARRANTY OF MERCHANTABILITY, FITNESS FOR A
     PARTICULAR PURPOSE OR NON INFRINGEMENT, ARE HEREBY EXCLUDED. SUN
     MICROSYSTEMS AND ITS LICENSORS SHALL NOT BE LIABLE FOR ANY DAMAGES
     SUFFERED BY LICENSEE AS A RESULT OF USING, MODIFYING OR DISTRIBUTING
     THE SOFTWARE OR ITS DERIVATIVES. IN NO EVENT WILL SUN MICROSYSTEMS OR
     ITS LICENSORS BE LIABLE FOR ANY LOST REVENUE, PROFIT OR DATA, OR FOR
     DIRECT, INDIRECT, SPECIAL, CONSEQUENTIAL, INCIDENTAL OR PUNITIVE
     DAMAGES, HOWEVER CAUSED AND REGARDLESS OF THE THEORY OF LIABILITY,
     ARISING OUT OF THE USE OF OR INABILITY TO USE SOFTWARE, EVEN IF SUN
     MICROSYSTEMS HAS BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.

     You acknowledge that this Software is not designed, licensed or
     intended for use in the design, construction, operation or maintenance
     of any nuclear facility.
-->
<xsl:stylesheet
  xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:fo="http://www.w3.org/1999/XSL/Format"
  version="1.0">

  <xsl:output method="xml"/>

  <xsl:strip-space elements="*"/>

  <xsl:param name="verbose" select="false()"/>

  <xsl:template match="/">
    <xsl:if test="$verbose">
      <xsl:message>Stripping all whitespace-only text nodes.</xsl:message>
      <xsl:message>Normalizing all remaining text nodes.</xsl:message>
    </xsl:if>
    <xsl:apply-templates/>
  </xsl:template>
  
  <xsl:template match="fo:list-item-label/text()">
    <xsl:if test="$verbose">
      <xsl:message>Correcting text child of fo:list-item-label by removing text</xsl:message>
    </xsl:if>
  </xsl:template>

  <xsl:template match="text()[. = '&#xA0;']">
    <xsl:if test="$verbose">
      <xsl:message>Removing "&amp;#xA0;".</xsl:message>
    </xsl:if>
  </xsl:template>

  <xsl:template match="text()">
    <xsl:value-of select="normalize-space()"/>
  </xsl:template>

  <xsl:template match="@border">
      <xsl:choose>
      <xsl:when test=". = 'solid 1pt'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>
        <xsl:attribute name="border-top-style">solid</xsl:attribute>
        <xsl:attribute name="border-bottom-style">solid</xsl:attribute>
        <xsl:attribute name="border-left-style">solid</xsl:attribute>
        <xsl:attribute name="border-right-style">solid</xsl:attribute>

        <xsl:attribute name="border-top-width">1pt</xsl:attribute>
        <xsl:attribute name="border-bottom-width">1pt</xsl:attribute>
        <xsl:attribute name="border-left-width">1pt</xsl:attribute>
        <xsl:attribute name="border-right-width">1pt</xsl:attribute>
      </xsl:when>
      <xsl:when test=". = 'blue,0.1pt,solid'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>

        <xsl:attribute name="border-top-color">blue</xsl:attribute>
        <xsl:attribute name="border-bottom-color">blue</xsl:attribute>
        <xsl:attribute name="border-left-color">blue</xsl:attribute>
        <xsl:attribute name="border-right-color">blue</xsl:attribute>

        <xsl:attribute name="border-top-style">solid</xsl:attribute>
        <xsl:attribute name="border-bottom-style">solid</xsl:attribute>
        <xsl:attribute name="border-left-style">solid</xsl:attribute>
        <xsl:attribute name="border-right-style">solid</xsl:attribute>

        <xsl:attribute name="border-top-width">0.1pt</xsl:attribute>
        <xsl:attribute name="border-bottom-width">0.1pt</xsl:attribute>
        <xsl:attribute name="border-left-width">0.1pt</xsl:attribute>
        <xsl:attribute name="border-right-width">0.1pt</xsl:attribute>
      </xsl:when>
      <xsl:when test=". = '3pt,black,solid'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>

        <xsl:attribute name="border-top-color">black</xsl:attribute>
        <xsl:attribute name="border-bottom-color">black</xsl:attribute>
        <xsl:attribute name="border-left-color">black</xsl:attribute>
        <xsl:attribute name="border-right-color">black</xsl:attribute>

        <xsl:attribute name="border-top-style">solid</xsl:attribute>
        <xsl:attribute name="border-bottom-style">solid</xsl:attribute>
        <xsl:attribute name="border-left-style">solid</xsl:attribute>
        <xsl:attribute name="border-right-style">solid</xsl:attribute>

        <xsl:attribute name="border-top-width">3pt</xsl:attribute>
        <xsl:attribute name="border-bottom-width">3pt</xsl:attribute>
        <xsl:attribute name="border-left-width">3pt</xsl:attribute>
        <xsl:attribute name="border-right-width">3pt</xsl:attribute>
      </xsl:when>
      <xsl:when test=". = 'black 1pt solid'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>

        <xsl:attribute name="border-top-color">black</xsl:attribute>
        <xsl:attribute name="border-bottom-color">black</xsl:attribute>
        <xsl:attribute name="border-left-color">black</xsl:attribute>
        <xsl:attribute name="border-right-color">black</xsl:attribute>

        <xsl:attribute name="border-top-style">solid</xsl:attribute>
        <xsl:attribute name="border-bottom-style">solid</xsl:attribute>
        <xsl:attribute name="border-left-style">solid</xsl:attribute>
        <xsl:attribute name="border-right-style">solid</xsl:attribute>

        <xsl:attribute name="border-top-width">1pt</xsl:attribute>
        <xsl:attribute name="border-bottom-width">1pt</xsl:attribute>
        <xsl:attribute name="border-left-width">1pt</xsl:attribute>
        <xsl:attribute name="border-right-width">1pt</xsl:attribute>
      </xsl:when>
      <xsl:when test=". = 'silver solid'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>

        <xsl:attribute name="border-top-color">silver</xsl:attribute>
        <xsl:attribute name="border-bottom-color">silver</xsl:attribute>
        <xsl:attribute name="border-left-color">silver</xsl:attribute>
        <xsl:attribute name="border-right-color">silver</xsl:attribute>

        <xsl:attribute name="border-top-style">solid</xsl:attribute>
        <xsl:attribute name="border-bottom-style">solid</xsl:attribute>
        <xsl:attribute name="border-left-style">solid</xsl:attribute>
        <xsl:attribute name="border-right-style">solid</xsl:attribute>
      </xsl:when>
      <xsl:otherwise>
        <xsl:attribute name="{name()}">
          <xsl:value-of select="."/>
        </xsl:attribute>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="@border-color">
    <xsl:choose>
      <xsl:when test=". = '#000000 #000000 #000000 #000000'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>
        <xsl:attribute name="border-top-color">#000000</xsl:attribute>
        <xsl:attribute name="border-bottom-color">#000000</xsl:attribute>
        <xsl:attribute name="border-left-color">#000000</xsl:attribute>
        <xsl:attribute name="border-right-color">#000000</xsl:attribute>
      </xsl:when>
      <xsl:when test=". = '#000000 #000000 #000099 #000000'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>
        <xsl:attribute name="border-top-color">#000000</xsl:attribute>
        <xsl:attribute name="border-bottom-color">#000000</xsl:attribute>
        <xsl:attribute name="border-left-color">#000099</xsl:attribute>
        <xsl:attribute name="border-right-color">#000000</xsl:attribute>
      </xsl:when>
      <xsl:when test=". = '#000000 #000000 #ff0000 #000000'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>
        <xsl:attribute name="border-top-color">#000000</xsl:attribute>
        <xsl:attribute name="border-bottom-color">#000000</xsl:attribute>
        <xsl:attribute name="border-left-color">#ff0000</xsl:attribute>
        <xsl:attribute name="border-right-color">#000000</xsl:attribute>
      </xsl:when>
      <xsl:when test=". = '#000000 #ff0000 #000000 #000000'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>
        <xsl:attribute name="border-top-color">#000000</xsl:attribute>
        <xsl:attribute name="border-bottom-color">#ff0000</xsl:attribute>
        <xsl:attribute name="border-left-color">#000000</xsl:attribute>
        <xsl:attribute name="border-right-color">#000000</xsl:attribute>
      </xsl:when>
      <xsl:when test=". = '#000099 #000000 #000099 #000000'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>
        <xsl:attribute name="border-top-color">#000099</xsl:attribute>
        <xsl:attribute name="border-bottom-color">#000000</xsl:attribute>
        <xsl:attribute name="border-left-color">#000099</xsl:attribute>
        <xsl:attribute name="border-right-color">#000000</xsl:attribute>
      </xsl:when>
      <xsl:when test=". = '#000099 #000099 #000099 #000099'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>
        <xsl:attribute name="border-top-color">#000099</xsl:attribute>
        <xsl:attribute name="border-bottom-color">#000099</xsl:attribute>
        <xsl:attribute name="border-left-color">#000099</xsl:attribute>
        <xsl:attribute name="border-right-color">#000099</xsl:attribute>
      </xsl:when>
      <xsl:when test=". = '#990099 #000000 #990099 #000000'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>
        <xsl:attribute name="border-top-color">#990099</xsl:attribute>
        <xsl:attribute name="border-bottom-color">#000000</xsl:attribute>
        <xsl:attribute name="border-left-color">#990099</xsl:attribute>
        <xsl:attribute name="border-right-color">#000000</xsl:attribute>
      </xsl:when>
      <xsl:when test=". = '#ff0000 #ff0000 #ff0000 #ff0000'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>
        <xsl:attribute name="border-top-color">#ff0000</xsl:attribute>
        <xsl:attribute name="border-bottom-color">#ff0000</xsl:attribute>
        <xsl:attribute name="border-left-color">#ff0000</xsl:attribute>
        <xsl:attribute name="border-right-color">#ff0000</xsl:attribute>
      </xsl:when>
      <xsl:otherwise>
        <xsl:attribute name="{name()}">
          <xsl:value-of select="."/>
        </xsl:attribute>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="@border-bottom|@border-top|@border-right|@border-left">
    <xsl:choose>
      <xsl:when test=". = 'solid'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>
        <xsl:attribute name="{name()}-style">solid</xsl:attribute>
      </xsl:when>
      <xsl:when test=". = 'solid thin'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>
        <xsl:attribute name="{name()}-style">solid</xsl:attribute>
        <xsl:attribute name="{name()}-width">thin</xsl:attribute>
      </xsl:when>
      <xsl:when test=". = '0.5pt,red,solid'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>
        <xsl:attribute name="{name()}-color">red</xsl:attribute>
        <xsl:attribute name="{name()}-style">solid</xsl:attribute>
        <xsl:attribute name="{name()}-width">0.5pt</xsl:attribute>
      </xsl:when>
      <xsl:when test=". = '0.5pt,blue,solid'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>
        <xsl:attribute name="{name()}-color">blue</xsl:attribute>
        <xsl:attribute name="{name()}-style">solid</xsl:attribute>
        <xsl:attribute name="{name()}-width">0.5pt</xsl:attribute>
      </xsl:when>
      <xsl:otherwise>
        <xsl:attribute name="{name()}">
          <xsl:value-of select="."/>
        </xsl:attribute>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="@border-style">
    <xsl:choose>
      <xsl:when test=". = 'solid' or . = 'solid solid solid solid'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>
        <xsl:attribute name="border-top-style">solid</xsl:attribute>
        <xsl:attribute name="border-bottom-style">solid</xsl:attribute>
        <xsl:attribute name="border-left-style">solid</xsl:attribute>
        <xsl:attribute name="border-right-style">solid</xsl:attribute>
      </xsl:when>
      <xsl:when test=". = 'solid solid dashed solid'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>
        <xsl:attribute name="border-top-style">solid</xsl:attribute>
        <xsl:attribute name="border-bottom-style">solid</xsl:attribute>
        <xsl:attribute name="border-left-style">dashed</xsl:attribute>
        <xsl:attribute name="border-right-style">solid</xsl:attribute>
      </xsl:when>
      <xsl:when test=". ='dashed solid solid solid'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>
        <xsl:attribute name="border-top-style">dashed</xsl:attribute>
        <xsl:attribute name="border-bottom-style">solid</xsl:attribute>
        <xsl:attribute name="border-left-style">solid</xsl:attribute>
        <xsl:attribute name="border-right-style">solid</xsl:attribute>
      </xsl:when>
      <xsl:otherwise>
        <xsl:attribute name="{name()}">
          <xsl:value-of select="."/>
        </xsl:attribute>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="@border-width">
    <xsl:choose>
      <xsl:when test=". = '0.0pt 0.0pt 0.0pt 0.0pt'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>
        <xsl:attribute name="border-top-width">0.0pt</xsl:attribute>
        <xsl:attribute name="border-bottom-width">0.0pt</xsl:attribute>
        <xsl:attribute name="border-left-width">0.0pt</xsl:attribute>
        <xsl:attribute name="border-right-width">0.0pt</xsl:attribute>
      </xsl:when>
      <xsl:when test=". = '0.0pt 0.0pt 0.5pt 0.0pt'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>
        <xsl:attribute name="border-top-width">0.0pt</xsl:attribute>
        <xsl:attribute name="border-bottom-width">0.0pt</xsl:attribute>
        <xsl:attribute name="border-left-width">0.5pt</xsl:attribute>
        <xsl:attribute name="border-right-width">0.0pt</xsl:attribute>
      </xsl:when>
      <xsl:when test=". = '0.0pt 1.0pt 0.0pt 0.0pt'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>
        <xsl:attribute name="border-top-width">0.0pt</xsl:attribute>
        <xsl:attribute name="border-bottom-width">1.0pt</xsl:attribute>
        <xsl:attribute name="border-left-width">0.0pt</xsl:attribute>
        <xsl:attribute name="border-right-width">0.0pt</xsl:attribute>
      </xsl:when>
      <xsl:when test=". = '0.0pt 0.0pt 1.0pt 0.0pt'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>
        <xsl:attribute name="border-top-width">0.0pt</xsl:attribute>
        <xsl:attribute name="border-bottom-width">0.0pt</xsl:attribute>
        <xsl:attribute name="border-left-width">1.0pt</xsl:attribute>
        <xsl:attribute name="border-right-width">0.0pt</xsl:attribute>
      </xsl:when>
      <xsl:when test=". = '0.25pt 0.0pt 0.0pt 0.0pt'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>
        <xsl:attribute name="border-top-width">0.25pt</xsl:attribute>
        <xsl:attribute name="border-bottom-width">0.0pt</xsl:attribute>
        <xsl:attribute name="border-left-width">0.0pt</xsl:attribute>
        <xsl:attribute name="border-right-width">0.0pt</xsl:attribute>
      </xsl:when>
      <xsl:when test=". = '0.25pt 0.0pt 0.25pt 0.0pt'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>
        <xsl:attribute name="border-top-width">0.25pt</xsl:attribute>
        <xsl:attribute name="border-bottom-width">0.0pt</xsl:attribute>
        <xsl:attribute name="border-left-width">0.25pt</xsl:attribute>
        <xsl:attribute name="border-right-width">0.0pt</xsl:attribute>
      </xsl:when>
      <xsl:when test=". = '0.25pt 0.25pt 0.25pt 0.25pt'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>
        <xsl:attribute name="border-top-width">0.25pt</xsl:attribute>
        <xsl:attribute name="border-bottom-width">0.25pt</xsl:attribute>
        <xsl:attribute name="border-left-width">0.25pt</xsl:attribute>
        <xsl:attribute name="border-right-width">0.25pt</xsl:attribute>
      </xsl:when>
      <xsl:when test=". = '0.5pt 0.0pt 0.0pt 0.0pt'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>
        <xsl:attribute name="border-top-width">0.5pt</xsl:attribute>
        <xsl:attribute name="border-bottom-width">0.0pt</xsl:attribute>
        <xsl:attribute name="border-left-width">0.0pt</xsl:attribute>
        <xsl:attribute name="border-right-width">0.0pt</xsl:attribute>
      </xsl:when>
      <xsl:otherwise>
        <xsl:attribute name="{name()}">
          <xsl:value-of select="."/>
        </xsl:attribute>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="@display-align[.='middle']">
    <xsl:if test="$verbose">
      <xsl:message>Correcting 'display-align="middle"' to 'display-align="center"'.</xsl:message>
    </xsl:if>
    <xsl:attribute name="{name()}">center</xsl:attribute>
  </xsl:template>


  <xsl:template match="@height">
    <xsl:if test="$verbose">
      <xsl:message>Removing 'height' property.</xsl:message>
    </xsl:if>
  </xsl:template>

  <xsl:template match="@padding">
    <xsl:choose>
      <xsl:when test=". = '.125in'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>
        <xsl:attribute name="padding-top">.125in</xsl:attribute>
        <xsl:attribute name="padding-bottom">.125in</xsl:attribute>
        <xsl:attribute name="padding-left">.125in</xsl:attribute>
        <xsl:attribute name="padding-right">.125in</xsl:attribute>
      </xsl:when>
      <!--
      <xsl:when test=". = '.25in'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>
        <xsl:attribute name="padding-top">.25in</xsl:attribute>
        <xsl:attribute name="padding-bottom">.25in</xsl:attribute>
        <xsl:attribute name="padding-left">.25in</xsl:attribute>
        <xsl:attribute name="padding-right">.25in</xsl:attribute>
      </xsl:when>
      -->
      <xsl:when test=". = '0.5pt 0.0pt 0.0pt 0.0pt'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>
        <xsl:attribute name="padding-top">0.5pt</xsl:attribute>
        <xsl:attribute name="padding-bottom">0.0pt</xsl:attribute>
        <xsl:attribute name="padding-left">0.0pt</xsl:attribute>
        <xsl:attribute name="padding-right">0.0pt</xsl:attribute>
      </xsl:when>
      <xsl:when test=". = '2.0pt' or . = '2pt'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>
        <xsl:attribute name="padding-top">2.0pt</xsl:attribute>
        <xsl:attribute name="padding-bottom">2.0pt</xsl:attribute>
        <xsl:attribute name="padding-left">2.0pt</xsl:attribute>
        <xsl:attribute name="padding-right">2.0pt</xsl:attribute>
      </xsl:when>
      <xsl:when test=". = '2.0pt 2.0pt 0.0pt 2.0pt'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>
        <xsl:attribute name="padding-top">2.0pt</xsl:attribute>
        <xsl:attribute name="padding-bottom">2.0pt</xsl:attribute>
        <xsl:attribute name="padding-left">0.0pt</xsl:attribute>
        <xsl:attribute name="padding-right">2.0pt</xsl:attribute>
      </xsl:when>
      <xsl:when test=". = '2.85pt 2.0pt 0.0pt 2.0pt'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>
        <xsl:attribute name="padding-top">2.85pt</xsl:attribute>
        <xsl:attribute name="padding-bottom">2.0pt</xsl:attribute>
        <xsl:attribute name="padding-left">0.0pt</xsl:attribute>
        <xsl:attribute name="padding-right">2.0pt</xsl:attribute>
      </xsl:when>
      <xsl:when test=". = '3pt'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>
        <xsl:attribute name="padding-top">3pt</xsl:attribute>
        <xsl:attribute name="padding-bottom">3pt</xsl:attribute>
        <xsl:attribute name="padding-left">3pt</xsl:attribute>
        <xsl:attribute name="padding-right">3pt</xsl:attribute>
      </xsl:when>
      <xsl:when test=". = '6pt'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>
        <xsl:attribute name="padding-top">6pt</xsl:attribute>
        <xsl:attribute name="padding-bottom">6pt</xsl:attribute>
        <xsl:attribute name="padding-left">6pt</xsl:attribute>
        <xsl:attribute name="padding-right">6pt</xsl:attribute>
      </xsl:when>
      <xsl:when test=". = '24pt'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>
        <xsl:attribute name="padding-top">24pt</xsl:attribute>
        <xsl:attribute name="padding-bottom">24pt</xsl:attribute>
        <xsl:attribute name="padding-left">24pt</xsl:attribute>
        <xsl:attribute name="padding-right">24pt</xsl:attribute>
      </xsl:when>
      <xsl:when test=". = '.3em'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>
        <xsl:attribute name="padding-top">.3em</xsl:attribute>
        <xsl:attribute name="padding-bottom">.3em</xsl:attribute>
        <xsl:attribute name="padding-left">.3em</xsl:attribute>
        <xsl:attribute name="padding-right">.3em</xsl:attribute>
      </xsl:when>
      <xsl:when test=". = '3.95pt 2.0pt 2.0pt 0.0pt'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>
        <xsl:attribute name="padding-top">3.95pt</xsl:attribute>
        <xsl:attribute name="padding-bottom">2.0pt</xsl:attribute>
        <xsl:attribute name="padding-left">2.0pt</xsl:attribute>
        <xsl:attribute name="padding-right">0.0pt</xsl:attribute>
      </xsl:when>
      <xsl:when test=". = '8.5pt 0.0pt 11.35pt 11.35pt'">
	<xsl:if test="$verbose">
	  <xsl:message>Rewriting '<xsl:value-of select="name()"/>="<xsl:value-of select="."/>"' as multiple properties.</xsl:message>
	</xsl:if>
        <xsl:attribute name="padding-top">8.5pt</xsl:attribute>
        <xsl:attribute name="padding-bottom">0.0pt</xsl:attribute>
        <xsl:attribute name="padding-left">11.35pt</xsl:attribute>
        <xsl:attribute name="padding-right">11.35pt</xsl:attribute>
      </xsl:when>
      <xsl:otherwise>
        <xsl:attribute name="{name()}">
          <xsl:value-of select="."/>
        </xsl:attribute>
      </xsl:otherwise>
    </xsl:choose>
  </xsl:template>

  <xsl:template match="@*">
    <xsl:attribute name="{name()}">
      <xsl:value-of select="."/>
    </xsl:attribute>
  </xsl:template>

  <xsl:template match="fo:block-container[@reference-orientation]">
    <xsl:if test="$verbose">
      <xsl:message>Removing 'fo:block-container' with unsupported 'reference-orientation' property.</xsl:message>
    </xsl:if>
  </xsl:template>

  <xsl:template match="fo:block-container/text()">
    <xsl:if test="$verbose">
      <xsl:message>Correcting 'fo:block-container' containing only text.</xsl:message>
    </xsl:if>
    <fo:block>
      <xsl:copy/>
    </fo:block>
  </xsl:template>

  <xsl:template match="fo:external-graphic[contains(@src, '.gif')]">
    <xsl:if test="$verbose">
      <xsl:message>Removing 'fo:external-graphic' with unsupported format.</xsl:message>
    </xsl:if>
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="fo:float">
    <xsl:if test="$verbose">
      <xsl:message>Removing unsupported 'fo:float'.</xsl:message>
    </xsl:if>
    <xsl:apply-templates/>
  </xsl:template>

  <xsl:template match="fo:footnote">
    <xsl:if test="$verbose">
      <xsl:message>Removing unsupported 'fo:footnote'.</xsl:message>
    </xsl:if>
  </xsl:template>

  <xsl:template match="fo:inline-container/fo:block">
    <xsl:if test="$verbose">
      <xsl:message>Removing unsupported 'fo:block' in 'fo:inline-container'.</xsl:message>
    </xsl:if>
  </xsl:template>

  <xsl:template match="fo:simple-page-master[name(child::*[1]) = 'fo:region-before' and
    name(child::*[2]) = 'fo:region-after' and
		name(child::*[3]) = 'fo:region-body']">
    <xsl:if test="$verbose">
      <xsl:message>Correcting 'fo:region-before', 'fo:region-after', and 'fo:region-body' order.</xsl:message>
    </xsl:if>
    <xsl:copy>
      <xsl:apply-templates select="@*"/>
      <xsl:apply-templates select="fo:region-body"/>
      <xsl:apply-templates select="fo:region-before"/>
      <xsl:apply-templates select="fo:region-after"/>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="fo:root[name(child::*[1]) = 'fo:declarations' and
    name(child::*[2]) = 'fo:layout-master-set']">
    <xsl:if test="$verbose">
      <xsl:message>Correcting 'fo:declarations' and 'fo:layout-master-set' order.</xsl:message>
    </xsl:if>
    <xsl:copy>
      <xsl:apply-templates select="@*"/>
      <xsl:apply-templates select="fo:layout-master-set"/>
      <xsl:apply-templates select="fo:declarations"/>
      <xsl:apply-templates select="fo:page-sequence"/>
    </xsl:copy>
  </xsl:template>
  
  <xsl:template match="fo:static-content">
    <xsl:if test="$verbose">
      <xsl:message>Removing unsupported 'fo:static-content'.</xsl:message>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="fo:table[@table-layout='fixed' and not(@width)]">
    <xsl:if test="$verbose">
      <xsl:message>Auto table layout unsupported. Adding 'width' property.</xsl:message>
    </xsl:if>
    <xsl:copy>
      <xsl:attribute name="width">100%</xsl:attribute>
      <xsl:apply-templates select="@*"/>
      <xsl:apply-templates select="node()"/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="fo:table[not(@table-layout) and @width]">
    <xsl:if test="$verbose">
      <xsl:message>Auto table layout unsupported. Adding 'table-layout' property.</xsl:message>
    </xsl:if>
    <xsl:copy>
      <xsl:attribute name="table-layout">fixed</xsl:attribute>
      <xsl:apply-templates select="@*"/>
      <xsl:apply-templates select="node()"/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="fo:table[not(@table-layout) and not(@width)]">
    <xsl:if test="$verbose">
      <xsl:message>Auto table layout unsupported. Adding 'table-layout' and 'width' properties.</xsl:message>
    </xsl:if>
    <xsl:copy>
      <xsl:attribute name="table-layout">fixed</xsl:attribute>
      <xsl:attribute name="width">
        <xsl:choose>
          <xsl:when test="count(fo:table-column/@column-width) != 0">
	    <xsl:if test="$verbose">
	      <xsl:message>Computing table-width from 'fo:table-column/@column-width' properties.</xsl:message>
	    </xsl:if>
            <xsl:for-each select="fo:table-column/@column-width">
              <xsl:value-of select="."/>
              <xsl:if test="position() != last()">
                <xsl:text> + </xsl:text>
              </xsl:if>
            </xsl:for-each>
          </xsl:when>
          <xsl:otherwise>
	    <xsl:if test="$verbose">
	      <xsl:message>Using 'width="100%"'.</xsl:message>
	    </xsl:if>
            <xsl:text>100%</xsl:text>
          </xsl:otherwise>
        </xsl:choose>
      </xsl:attribute>
      <xsl:apply-templates select="@*"/>
      <xsl:apply-templates select="node()"/>
    </xsl:copy>
  </xsl:template>

  <xsl:template match="fo:wrapper[fo:list-item]">
    <xsl:if test="$verbose">
      <xsl:message>Removing 'fo:wrapper' used in an unsupported context.</xsl:message>
    </xsl:if>
    <xsl:apply-templates/>
  </xsl:template>
  
  <xsl:template match="fo:wrapper[count(node()) = 0]">
    <xsl:if test="$verbose">
      <xsl:message>Removing empty 'fo:wrapper'.</xsl:message>
    </xsl:if>
  </xsl:template>
  
  <xsl:template match="*">
    <xsl:copy>
      <xsl:apply-templates select="@*"/>
      <xsl:apply-templates select="node()"/>
    </xsl:copy>
  </xsl:template>

</xsl:stylesheet>
